/**
**************************************************************************
* @file tester_func.h
* @brief Заголовочный файл для функций МЗКТ.
**************************************************************************
@details

*************************************************************************/

	
#ifndef _TESTER_ADC_FUNC_H_
#define _TESTER_ADC_FUNC_H_

#include "mylibs_include.h"

typedef uint16_t ADC_BuffTypeDef[ADC_BUFF_SIZE];

typedef enum
{
  ADC_DISABLED              = 0,
  WAIT_FOR_FIRST_HALF       = 1,
  PROCESSING_FIRST_HALF     = 2,
  WAIT_FOR_SECOND_HALF      = 3,
  PROCESSING_SECOND_HALF    = 4,
  ADC_DONE                  = 5,
}ADC_ChannelState;

typedef struct
{
  uint16_t        ADC_zero;
  uint16_t        ADC_calibr;
  float           U_step;
  unsigned        pulse_width; 
  unsigned        expected_pulse_sign; 
}ADC_ParamsTypeDef;
typedef struct
{  
  float           U_Current;
  uint16_t        ADC_Filtered; 
  ADC_BuffTypeDef ADC_Buff;
  
  uint16_t        ADC_Max;
  uint16_t        ADC_Min;
  unsigned        pulse_peak_ind; 
  
  
  /* Parameter for calc voltage */
  ADC_ParamsTypeDef s;
  
   
  
  ADC_ChannelState state;
  
  ADCChannel_TrackerTypeDef   adc_ch_err;
}TESTER_ADCChannelTypeDef;

typedef struct
{
  TESTER_ADCChannelTypeDef chAdc;
  
  ADC_HandleTypeDef *hadc; 

//  TESTER_ADCFilterTypeDef filter;
  
  struct
  {
    
    unsigned adc_running:1;
    unsigned enable_dmaBuffTransfer:1;
    unsigned dmaBufferHalfDone:2;
    unsigned dmaHandlerbusy:1;
  }f;
  ADC_TrackerTypeDef adc_err;
}TESTER_ADCTypeDef;


extern TESTER_ADCTypeDef tester_adc;
extern uint16_t ADC_DMA_Buff[ADC_DMA_BUFF_SIZE];
extern uint16_t ADC_DMA_HalfBuff[ADC_DMA_BUFF_SIZE/2];

void TESTER_ADC_StructInit(TESTER_ADCTypeDef *adc);

void ADC_DMA_Handler(TESTER_ADCTypeDef *adc);

void ADC_DMA_ProcessBufferForPeak(TESTER_ADCChannelTypeDef *adc_fc);
void ADC_DMA_Channels_Prepare(TESTER_ADCChannelTypeDef *adc_fuel_cells, uint8_t MaskForChannels, uint8_t state);
HAL_StatusTypeDef ADC_DMA_StartRead(TESTER_ADCTypeDef *adc);
/* Считать буфер DMA и найти в нем пик (скачок напряжения) */
HAL_StatusTypeDef ADC_DMA_ReadForPeak(TESTER_ADCTypeDef *adc, uint32_t Timeout);
/* Считывать АЦП, который работает в континуес режиме */
HAL_StatusTypeDef ADC_ReadContinuous(TESTER_ADCTypeDef *adc, uint32_t buff_size, uint32_t tickstart, uint32_t timeout);

#ifndef printf_adc_result
#define printf_adc_result(...)
#endif
#ifndef printf_adc_processing
#define printf_adc_processing(...)
#endif

#ifndef printf_adc_err
#define printf_adc_err(...)
#endif

#ifndef printf_adc_it
#define printf_adc_it(...)
#endif

#ifndef printf_adc
#define printf_adc(...)
#endif
#endif //_TESTER_ADC_FUNC_H_

