/**
**************************************************************************
* @file tester_config.h
* @brief Конфигурации для тестера диодов
**************************************************************************
* @defgroup    TESTER_CONFIGS    Configs for project
* @brief       Конфигурации для тестера диодов
* @details     
@{
*************************************************************************/
#ifndef _TESTER_CONFIG_H_
#define _TESTER_CONFIG_H_

#define TESTER_PULES_EXPETCED_WIDTH   4   ///< Предполагаемая длительность пика в отчетах ацп


/** 
	* @addtogroup TESTER_POWER_SW_CONFIG   Configs for switches for power
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации для ключей питания
	@{
	*/
  
/* Состояния ключей для подключения и откючения питания */
#define POWER_CONNECT       1   ///< Питание подключено в данном состоянии пина
#define POWER_DISCONNECT    0   ///< Питание отключено в данном состоянии пина

/* Ключи для подключения положительного питания к диоду */
#define SWITCH_DC_POSITIVE_1_Port   GPIOC         ///< Порт пина первого ключа для положительного питания
#define SWITCH_DC_POSITIVE_1_Pin    GPIO_PIN_13   ///< Пин первого ключа для положительного питания
#define SWITCH_DC_POSITIVE_2_Port   GPIOA         ///< Порт пина второго ключа для положительного питания
#define SWITCH_DC_POSITIVE_2_Pin    GPIO_PIN_11   ///< Пин второго ключа для положительного питания


/* Ключи для подключения отрицательного питания к диоду */
#define SWITCH_DC_NEGATIVE_1_Port   GPIOC         ///< Порт пина первого ключа для отрицательного питания
#define SWITCH_DC_NEGATIVE_1_Pin    GPIO_PIN_13   ///< Пин первого ключа для отрицательного питания
#define SWITCH_DC_NEGATIVE_2_Port   GPIOA         ///< Порт пина второго ключа для отрицательного питания
#define SWITCH_DC_NEGATIVE_2_Pin    GPIO_PIN_11   ///< Пин первого ключа для отрицательного питания

/** TESTER_POWER_SW_CONFIG
  * @}
  */
  
/** 
	* @addtogroup TESTER_ADC_CONFIG   Configs for ADC
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации для АЦП
	@{
	*/
  
#define ADC_BUFF_SIZE                           (200)               ///< Размер буфера АЦП
#define ADC_DMA_BUFF_SIZE                       (ADC_BUFF_SIZE)     ///< Размер буфера ДМА (1 канал)

#define ADC_U_CALIBR                            ((float)3.0)        ///< Калибровочное напряжение
#define ADC_VALUE_CALIBR                        (0xFFF)             ///< Значение АЦП при калибровочном напряжении @ref ADC_U_CALIBR
#define ADC_VALUE_ZERO                          (0)                 ///< Значение АЦП при нулевом напряжении
  
#define ADC_READ_TIMEOUT_MS                     20                  ///< Таймаут на ожидание разрядки конденсатора

/** TESTER_ADC_CONFIG
  * @}
  */


/** TESTER_CONFIGS
  * @}
  */

#endif //_TESTER_CONFIG_H_

