#include "tester_func.h"

TESTER_TestHandleTypeDef hTestDiode;

/**s
  * @brief    Инициализация хендла тестера диодов
  */
void TESTER_HandleInit(TESTER_TestHandleTypeDef *htest)
{
  htest->adc = &tester_adc;
  
  TESTER_ADC_StructInit(htest->adc);
  
  htest->DCNegSw.SW1_Port = SWITCH_DC_NEGATIVE_1_Port;
  htest->DCNegSw.SW2_Port = SWITCH_DC_NEGATIVE_2_Port;
  htest->DCNegSw.SW1_Pin  = SWITCH_DC_NEGATIVE_1_Pin;
  htest->DCNegSw.SW2_Pin  = SWITCH_DC_NEGATIVE_2_Pin;
  
  
  htest->DCPosSw.SW1_Port = SWITCH_DC_POSITIVE_1_Port;
  htest->DCPosSw.SW2_Port = SWITCH_DC_POSITIVE_2_Port;
  htest->DCPosSw.SW1_Pin  = SWITCH_DC_POSITIVE_1_Pin;
  htest->DCPosSw.SW2_Pin  = SWITCH_DC_POSITIVE_2_Pin;
}


/**
  * @brief    Тест диодов: подключение прямого напряжения
  */
void TESTER_TestDiode_PositivePower(TESTER_TestHandleTypeDef *htest)
{
  /* Задержка, перед началом работы */
  TESTER_Delay(&htest->SwTimings.ticks_before_test);
  
  /* Включение континиус АЦП */
  HAL_ADC_Start(htest->adc->hadc);  
  
  /* Подкючение питания к диоду */
  TESTER_Connect_Power(&htest->DCPosSw);
  
  
  /* Подключение питания на определенное время */
  uint32_t tickstart = HAL_GetTick();
  HAL_StatusTypeDef res;
  while(1)
  {
    /* Считывание АЦП пока таймаут истечет */
    res = ADC_ReadContinuous(htest->adc, htest->continuous_buff_size, tickstart, htest->SwTimings.msticks_for_positive_dc);
    
    if(res != HAL_OK)
      break;
  }
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Power(&htest->DCPosSw);
  
  /* Выключение континиус АЦП */
  HAL_ADC_Stop(htest->adc->hadc);
  
}

/**
  * @brief    Тест диодов: подключение обратного напряжения
  */
void TESTER_TestDiode_NegativePower(TESTER_TestHandleTypeDef *htest)
{
  /* Задержка, перед началом работы */
  TESTER_Delay(&htest->SwTimings.ticks_before_test);
  
  /* Включение АЦП */
  ADC_DMA_StartRead(htest->adc);
  
  /* Задержка, перед предполагаемым скачком */
  TESTER_Delay(&htest->SwTimings.ticks_before_go_to_peak);
  
  /* Подкючение питания к диоду */
  TESTER_Connect_Power(&htest->DCNegSw);
  
  /* Подключение питания на определенное время */
  TESTER_Delay(&htest->SwTimings.ticks_before_disconnect);
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Power(&htest->DCNegSw);
  
  /* Обработка DMA */
  ADC_DMA_ReadForPeak(htest->adc, ADC_READ_TIMEOUT_MS);
  
}


/**
  * @brief    Тест диодов: подключение прямого, а потом обратного напряжения
  */
void TESTER_TestDiode_PosNegPower(TESTER_TestHandleTypeDef *htest)
{ 
  /* Задержка, перед началом работы */
  TESTER_Delay(&htest->SwTimings.ticks_before_test);
  
  /* Подкючение питания к диоду */
  TESTER_Connect_Power(&htest->DCPosSw);
  
  /* Подкючение положительного питания к диоду */
  msDelay(htest->SwTimings.msticks_for_positive_dc);
  
  /* Включение АЦП */
  ADC_DMA_StartRead(htest->adc);
  
  /* Ожидается задержка, перед предполагаемым скачком */
  TESTER_Delay(&htest->SwTimings.ticks_before_go_to_peak);
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Power(&htest->DCPosSw);
  TESTER_Connect_Power(&htest->DCNegSw);
  
  /* Подключение отрицательного питания на определенное время */
  TESTER_Delay(&htest->SwTimings.ticks_before_disconnect);
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Power(&htest->DCNegSw);
  
  /* Обработка DMA */
  ADC_DMA_ReadForPeak(htest->adc, ADC_READ_TIMEOUT_MS);
  
}

/**
  * @brief    Подключить питание с помощью ключей (пинов) в TESTER_PowerSwitchTypeDef
  */
void TESTER_Connect_Power(TESTER_PowerSwitchTypeDef *DCSw)
{
  HAL_GPIO_WritePin(DCSw->SW1_Port, DCSw->SW1_Pin, POWER_CONNECT);
  HAL_GPIO_WritePin(DCSw->SW2_Port, DCSw->SW2_Pin, POWER_CONNECT);
}
/**
  * @brief    Отключить питание с помощью ключей (пинов) в TESTER_PowerSwitchTypeDef
  */
void TESTER_Disconnect_Power(TESTER_PowerSwitchTypeDef *DCSw)
{
  HAL_GPIO_WritePin(DCSw->SW1_Port, DCSw->SW1_Pin, POWER_DISCONNECT);
  HAL_GPIO_WritePin(DCSw->SW2_Port, DCSw->SW2_Pin, POWER_DISCONNECT);
}

/**
  * @brief    Формирование задержки (в тиках или миллисекундная)
  */
void TESTER_Delay(TESTER_TicksDelayTypeDef *tickdelay)
{
  /* Задержка, перед подключением питания */
  if(tickdelay->msdelay == 0) // если миллисекундная задержка выключена
  {
    /* Задержка, в тиках  */
    for(int i = 0; i < tickdelay->ticks; i++);
  }
  else // если миллисекундная задержка включена
  {
    /* Задержка, в миллисекундах */
    msDelay(tickdelay->ticks);
  }
}