/**
**************************************************************************
* @file tester_func.h
* @brief Заголовочный файл для функций МЗКТ.
**************************************************************************
@details

*************************************************************************/

	
#ifndef _TESTER_FUNC_H_
#define _TESTER_FUNC_H_

#include "tester_adc_func.h"


/**
  * @brief Структура для пинов, которые отвечают за ключи, которые подключают питание
  */
typedef struct
{
  GPIO_TypeDef *SW1_Port;   ///< Порт пина первого ключа для питания
  uint32_t      SW1_Pin;    ///< Пин первого ключа для питания
  
  GPIO_TypeDef *SW2_Port;   ///< Порт пина второго ключа для питания
  uint32_t      SW2_Pin;    ///< Пин второго ключа для питания
}TESTER_PowerSwitchTypeDef;


/**
  * @brief Структура для тайминга с флагом миллисекундой или тиковой (пустой for) задержкой
  */
typedef struct
{
  unsigned  msdelay:1;
  uint32_t  ticks;
}TESTER_TicksDelayTypeDef;

/**
  * @brief Структура с таймингами для тестера 
  */
typedef struct
{
  uint32_t                  msticks_for_positive_dc;    ///< миллисекундная задержка для положительного напряжения @ref TESTER_TestDiode_PositivePower
  TESTER_TicksDelayTypeDef  ticks_before_test;          ///< задержка перед началом тестирования (мс или тики for())
  TESTER_TicksDelayTypeDef  ticks_before_disconnect;    ///< задержка перед выключением питания (мс или тики for())
  TESTER_TicksDelayTypeDef  ticks_before_go_to_peak;    ///< задержка между включением АЦП и предполагаемым скачком напряжения, чтобы точно поймать его в буфере АЦП (мс или тики for())
}TESTER_SwitchTimingsTypeDef;

/**
  * @brief Структура-дескриптор для управления тестированием диодов
  */
typedef struct
{
  TESTER_ADCTypeDef *adc;                 ///< указатель на структуру АЦП тестера
  
  TESTER_SwitchTimingsTypeDef SwTimings;  ///< Тайминги для ключей
  
  TESTER_PowerSwitchTypeDef DCPosSw;      ///< Пины ключей для положительного источника
  TESTER_PowerSwitchTypeDef DCNegSw;      ///< Пины ключей для отрицательного источника
  
  uint32_t          continuous_buff_size;
  
  float             DiodeVolt;
}TESTER_TestHandleTypeDef;
extern TESTER_TestHandleTypeDef hTestDiode;



/* Инициализация хендла тестера диодов */
void TESTER_HandleInit(TESTER_TestHandleTypeDef *htest);

/* Тест диодов: подключение прямого напряжения */
void TESTER_TestDiode_PositivePower(TESTER_TestHandleTypeDef *htest);
/* Тест диодов: подключение обратного напряжения */
void TESTER_TestDiode_NegativePower(TESTER_TestHandleTypeDef *htest);
/* Тест диодов: подключение прямого, а потом обратного напряжения */
void TESTER_TestDiode_PosNegPower(TESTER_TestHandleTypeDef *htest);

/* Подключить питание с помощью ключей (пинов) в TESTER_PowerSwitchTypeDef */
void TESTER_Connect_Power(TESTER_PowerSwitchTypeDef *DCSw);
/* Отключить питание с помощью ключей (пинов) в TESTER_PowerSwitchTypeDef */
void TESTER_Disconnect_Power(TESTER_PowerSwitchTypeDef *DCSw);
/* Формирование задержки перед подключением питания (в тиках или миллисекундная) */
void TESTER_Delay(TESTER_TicksDelayTypeDef *tickdelay);
#endif //_TESTER_FUNC_H_

